/****************************************************************
**
** Implementation of GameBoard class, Qt tutorial 14
**
****************************************************************/

#include "gamebrd.h"

#include <qfont.h>
#include <qapplication.h>
#include <qlabel.h>
#include <qaccel.h>
#include <qpushbutton.h>
#include <qlcdnumber.h>
#include <qlayout.h>
#include <qvbox.h>
#include <qfile.h>
#include <qfiledialog.h>
#include <qtimer.h>
#include <qmessagebox.h>
#include <math.h>
//#include "lcdrange.h"


GameBoard::GameBoard( QWidget *parent, const char *name )
        : QWidget( parent, name )
{
		i0=0;
    i1=0;

    timer0 = new QTimer( this );
    connect( timer0, SIGNAL(timeout()), this, SLOT(showEvent0()) );
    //timer1->start( 100, FALSE ); 

    timer1 = new QTimer( this );
    connect( timer1, SIGNAL(timeout()), this, SLOT(showEvent1()) );
    //timer2->start( 50, FALSE ); 

    QPushButton *quit = new QPushButton( "&Quit", this, "quit" );
    quit->setFont( QFont( "Times", 18, QFont::Bold ) );
    connect( quit, SIGNAL(clicked()), qApp, SLOT(quit()) );

    QPushButton *start = new QPushButton( "&Start", this, "start" );
    start->setFont( QFont( "Times", 18, QFont::Bold ) );

    QPushButton *stop = new QPushButton( "S&top", this, "stop" );
    stop->setFont( QFont( "Times", 18, QFont::Bold ) );

		s0 = new Sector(this, "sector0");    
		s1 = new Sector(this, "sector1");
    connect( start, SIGNAL(clicked()), this, SLOT(startTimer()) );
    connect( stop, SIGNAL(clicked()), this, SLOT(stop()) );

    QGridLayout *grid = new QGridLayout( this, 3, 1, 10 );

    grid->setRowStretch( 1, 10 );


    QPushButton *fileButton1 = new QPushButton( "Trace File 1", this, "trace file 1" );
    QPushButton *fileButton2 = new QPushButton( "Trace File 2", this, "trace file 2" );

    connect( fileButton1, SIGNAL(clicked()), this, SLOT(readFile1()) );
    connect( fileButton2, SIGNAL(clicked()), this, SLOT(readFile2()) );

    fileLabel1 = new QLabel( "Trace file 1", this, "traceLabel1" );
    fileLabel2 = new QLabel( "Trace file 2", this, "traceLabel1" );

    QHBoxLayout *topBox = new QHBoxLayout;
		grid->addLayout(topBox, 0, 0);
		topBox->addWidget(fileButton1);
		topBox->addWidget(fileLabel1);
		topBox->addWidget(fileButton2);
		topBox->addWidget(fileLabel2);

    QHBoxLayout *midBox = new QHBoxLayout;
		grid->addLayout(midBox, 1, 0);
    midBox->addWidget(s0);
    midBox->addWidget(s1);


    QHBoxLayout *btmBox = new QHBoxLayout;
    grid->addLayout( btmBox, 2, 0 );
		btmBox->addWidget(start);
		btmBox->addWidget(stop);
		btmBox->addWidget(quit);
}

void GameBoard::startTimer(){
    timer0->start( 100, FALSE );
		timer1->start(110, FALSE); 
}

void GameBoard::showEvent0(){
  unsigned int next = i0 +1;
  if(next < s0->eventList.count()){
    if( s0->eventList[i0+1]->timeStamp < s1->eventList[i1+1]->timeStamp){
      if(i0==0){
        timer0->changeInterval(1000 * (int)floor(s0->eventList[next]->timeStamp - s0->eventList[i0]->timeStamp));
      }
	    else{
	      s0->showEvent(i0, 0);
        timer0->changeInterval(1000 * (int)floor(s0->eventList[next]->timeStamp - s0->eventList[i0]->timeStamp));
		  }
	    i0++;
		}
	}  
	else{
		timer0->stop();
  }
}

void GameBoard::showEvent1(){
  unsigned int next = i1 +1;
  if(next < s1->eventList.count()){
    if( s1->eventList[i1+1]->timeStamp < s0->eventList[i0+1]->timeStamp){
      if(i1==0){
        timer1->changeInterval(1000 * (int)floor(s1->eventList[next]->timeStamp - s1->eventList[i1]->timeStamp));
      }
  	  else{
  	    s1->showEvent(i1, 30);
        timer1->changeInterval(1000 * (int)floor(s1->eventList[next]->timeStamp - s1->eventList[i1]->timeStamp));
	  	}
	    i1++;
		}
	}  
	else{
		timer1->stop();
  }
}

void GameBoard::readFile1()
{
  QString fileName = QFileDialog::getOpenFileName(
                    "./",
                    "Images (*.xml *.png *.xpm *.jpg)",
                    this,
                    "open file dialog",
                    "Choose a file" );
	fileLabel1->setText(fileName);

  QFile xmlFile( fileName );
  if ( !xmlFile.open( IO_ReadOnly ) ) {
    QMessageBox::critical( 0
                         , tr( "Critical Error" )
                         , tr( "Cannot open file %1" ).arg( fileName )
                         );
    return;
  }
  if ( !domTree.setContent( &xmlFile ) ) {
    QMessageBox::critical( 0
                         , tr( "Critical Error" )
                         , tr( "Parsing error for file %1" ).arg( fileName )
                         );
    xmlFile.close();
    return;
  }
  xmlFile.close();
  QDomElement root = domTree.documentElement();
	s0->getTrace(root);
}

void GameBoard::readFile2()
{
  QString fileName = QFileDialog::getOpenFileName(
                    "./",
                    "Images (*.xml *.png *.xpm *.jpg)",
                    this,
                    "open file dialog",
                    "Choose a file" );
	fileLabel2->setText(fileName);

  QFile xmlFile( fileName );
  if ( !xmlFile.open( IO_ReadOnly ) ) {
    QMessageBox::critical( 0
                         , tr( "Critical Error" )
                         , tr( "Cannot open file %1" ).arg( fileName )
                         );
    return;
  }
  if ( !domTree.setContent( &xmlFile ) ) {
    QMessageBox::critical( 0
                         , tr( "Critical Error" )
                         , tr( "Parsing error for file %1" ).arg( fileName )
                         );
    xmlFile.close();
    return;
  }
  xmlFile.close();
  QDomElement root = domTree.documentElement();
	s1->getTrace(root);
}

void GameBoard::stop(){
	timer0->stop();
	i0=0;
	timer1->stop();
	i1=0;
}
void GameBoard::setText1(){
	i0 ++;
	s0->showEvent(i0, 0);
  //fileLabel1->setNum(i1);
}

void GameBoard::setText2(){
	i1 ++;
  timer1->changeInterval(50 * i1);
  fileLabel1->setNum(i1);
}
