#include "datastructure.h"
using namespace std;
/////////////////////////////////////////////
//	Velocity
/////////////////////////////////////////////
Velocity::Velocity(int vp, int ang){
  v = vp;
  angle = ang;
  vx = v * cos(angle*PI/180);
  vy = v * sin(angle*PI/180);
}

Velocity::Velocity(const Velocity& src){
  vx = src.vx;
  vy = src.vy;
  v = src.v;
  angle = src.angle;
}

Velocity& Velocity::operator=(const Velocity& src){
  vx = src.vx;
  vy = src.vy;
  v = src.v;
  angle = src.angle;      
  return *this;
}

void Velocity::computeAngle(){
	v=sqrt(pow(vx, 2.0)+pow(vy, 2.0));
  angle  = acos(vx/v)*180/PI;
  if(vx<0&&vy<0)
    angle += 90;
  if(vx>0 && vy<0)
    angle += 270.0;
}

void Velocity::print(){
  cout << "Velocity: ("<< v << ",[vx=" << vx << ", vy="<< vy << "]," << angle<< ")";
}

////////////////////////////////////////
//	Coordinate
////////////////////////////////////////
Coordinate::Coordinate(double coordX, double coordY){
  x = coordX;
  y = coordY;
}
Coordinate::Coordinate(const Coordinate& src){
  x = src.x;
  y = src.y;      
}
Coordinate& Coordinate::operator=(const Coordinate& src){
  x=src.x;
  y=src.y;
  return *this;
}
void Coordinate::print(){
  cout << "Coordinate: (" << x << "," << y << ")";
}

/////////////////////////////////////////////
//	Billiard
/////////////////////////////////////////////

//Constructors
Billiard::Billiard(Coordinate cd, Velocity vl){
		ballCounter ++;
  id = ballCounter;
  coord = cd;
  v = vl;
}  

//Copy Constructor
Billiard::Billiard(const Billiard& src){
  id = src.id;
  coord = src.coord;
  v = src.v;
}

//Assign operation
Billiard& Billiard::operator=(const Billiard& src){
  id = src.id;
  coord = src.coord;
  v = v;
  return *this;
}

void Billiard::print(){
  cout << "Billiard " << id  << " : ";
  coord.print();
  cout << "\t";
  v.print();
  cout << endl;
}

double Billiard::distance(const Billiard& dest){
  double dx=dest.coord.x-coord.x;
  double dy=dest.coord.y-coord.y;
	double dist=sqrt(pow(dx, 2.0)+pow(dy, 2.0));
  return dist;
}

double Billiard::angle(const Billiard& dest){
  double dx=dest.coord.x-coord.x;
  double dy=dest.coord.y-coord.y;
	double dist = distance(dest);
  double ang  = acos(dx/dist)*180/PI;
  if(dx<0&&dy>0)
    ang += 90;
  if(dx>0 && dy>0)
    ang += 270.0;
  return ang;
}

double Billiard::collisionVelocity(const Billiard& dest){
  double ang = angle(dest);
  double aDiff=ang-v.angle;
  return v.v*cos(aDiff*PI/180);
}

double Billiard::collisionVerticalVelocity(const Billiard& dest){
  double ang = angle(dest);
  double aDiff=ang-v.angle;
  return v.v*sin(aDiff*PI/180);
}

double Billiard::collisionTime(Billiard* pDest){
  double v1 = collisionVelocity(*pDest);
  double v2 = pDest->collisionVelocity(*this);
  
  cout << "ball " << id << ": v1= " << v1 << " a1= " << angle(*pDest) << " dist= " << distance(*pDest)<< endl;
  cout << "ball " << pDest->id << ": v2= " << v2 << " a2= " << pDest->angle(*this) << endl;
  if(v1+v2>0){
    double dist = distance(*pDest);
    return dist/(v1+v2);
  }
  return INF;
}

void Billiard::exchangeVelocity(Billiard* pDest){
	double vc1=collisionVelocity(*pDest);  
  double vv1=collisionVerticalVelocity(*pDest);
  double ang1=angle(*pDest);
  double vc2=pDest->collisionVelocity(*this);
  double vv2=pDest->collisionVerticalVelocity(*this);
  double ang2=pDest->angle(*this);
  
  v.vx = vc2*cos(ang2*PI/180)+vv1*cos((ang1-90)*PI/180);
  v.vy = vc2*sin(ang2*PI/180)+vv1*sin((ang1-90)*PI/180);
  v.v  = sqrt(pow(v.vx,2.0)+pow(v.vy,2.0));
  v.angle=acos(v.vx/v.v)*180/PI;

  pDest->v.vx = vc1*cos(ang1*PI/180)+vv2*cos((ang2-90)*PI/180);
  pDest->v.vy = vc1*sin(ang1*PI/180)+vv2*sin((ang2-90)*PI/180);
  pDest->v.v  = sqrt(pow(pDest->v.vx,2.0) + pow(pDest->v.vy,2.0));
  pDest->v.angle=acos(pDest->v.vx/pDest->v.v)*180/PI;

}
