#include <math.h>
#include <iostream>
#include <vector>
//#include <cstdlib>

using namespace std;

#define PI 3.14159265
#define INF 100000.0

/////////////////////////////////////////////
//	Velocity
/////////////////////////////////////////////
class Velocity{  
  public:
    double    v;
    double    angle;
    double    vx;
    double    vy;    

		//default constructor
    Velocity(){}
		//constructor with horizontal speed and vertical speed
    Velocity(int v, int angle);
		//copy constructor
    Velocity(const Velocity& src);
		//equality operator
    Velocity& operator=(const Velocity& src);
    void computeAngle();
		//print function
    void print();
};

/////////////////////////////////////////////
//	Coordinate
/////////////////////////////////////////////
class Coordinate{
  public:
    double x;
    double y;

    Coordinate(){}
    Coordinate(double coordX, double coordY);
    Coordinate(const Coordinate& src);
    Coordinate& operator=(const Coordinate& src);
    void print();
};

/////////////////////////////////////////////
//	Billiard
/////////////////////////////////////////////
class Billiard{
  public:
    int        id;
    Coordinate coord;
    Velocity   v;
    static int ballCounter;
    
    //Constructors
    Billiard(){}
    Billiard(Coordinate cd, Velocity vl);
    //Copy Constructor
    Billiard(const Billiard& src);
    double distance(const Billiard& dest);
    double angle(const Billiard& dest);
    double collisionVelocity(const Billiard& dest);
		double collisionVerticalVelocity(const Billiard& dest);
    double collisionTime(Billiard* pDest);
    //double bounceTime();
    void exchangeVelocity(Billiard* pDest);
    //Assign operation
    Billiard& operator=(const Billiard& src);
    void print();
    ~Billiard(){ballCounter--;}
};

/////////////////////////////////////////////
//	Event
/////////////////////////////////////////////
class Event{
  public:
    enum EventType { Empty, Initial, Collision, Arrival, Departure, Bounce};
    double     timeStamp;
    EventType  type;
    Billiard*  ball;
    Billiard*  ball1; //special for collision event
    vector<Billiard*> ballList;
    
    Event(){
			timeStamp=100000.0;
			type  = Empty;
			ball  = NULL;
      ball1 = NULL;
		}

    Event(double t, EventType eType, Billiard* b){
      timeStamp = t;
      type      = eType;
      ball      = b;
      ball1     = NULL;
    }

    Event(const Event& src){
			timeStamp = src.timeStamp;
      type      = src.type;
      ball      = src.ball;
      ball1     = src.ball1;
    }

		Event& operator=(const Event& src){
			timeStamp = src.timeStamp;
      type      = src.type;
      ball      = src.ball;
      ball1     = src.ball1;
      return *this;			
		}

    bool operator<(const Event& src){
			return (timeStamp < src.timeStamp);
    }
    bool operator==(const Event& src){
			return (timeStamp == src.timeStamp);
    }
/*
    void print(fstream fcout){
      if(type==Collision){
  	    fcout << "Collision Event : \t--time " << timeStamp << " type " << type << endl;
	    }
      else{
	      fcout << "Bounce Event : \t--time " << timeStamp << " type " << type << endl;
      }
      if(ball!=NULL){
        fcout << "\t--ball ";
				ball->print();
      }
      if(ball1!=NULL){
        fcout << "\t--ball1 ";
				ball1->print();
      }
    }
*/
    void print(){
      if(type==Collision){
  	    cout << "Collision Event : \t--time " << timeStamp << " type " << type << endl;
	    }
      else{
	      cout << "Bounce Event : \t--time " << timeStamp << " type " << type << endl;
      }
      if(ball!=NULL){
        cout << "\t--ball ";
				ball->print();
      }
      if(ball1!=NULL){
        cout << "\t--ball1 ";
				ball1->print();
      }
    }
    ~Event(){
     for (vector<Billiard*>::iterator p = ballList.begin(); p!=ballList.end(); p++)
       delete *p;
    }
};

