#include <iostream>
#include <fstream>
//#include <mpi.h>
#include <ctime>
#include <cstdlib>
#include <math.h>
//#include "datastructure.h"
#include "sector.h"

using namespace std;

class Util{
  public:
    static int vGenerator(int centerNum, int offset){
        return (rand()%(offset*2+1))-offset+centerNum;        
    }

    static int vGenerator1(int begin, int end){
        return (rand()%(end-begin))+begin;        
    }

    static int aGenerator(int begin, int end){
        int ang=(rand()%(end-begin))+begin;
        if(ang%90==0)
         ang+=1;
        return ang;
    }

    static int pGenerator(int d = 200){
      int p=d-1;
      return ( rand()%p + 1); //between 1 to 199
    }

    static int pGenerator1(int s = 0, int d = 200){
      int p=d - s -1;
      return ( rand()%p + 1 + s); //between 1 to 199
    }

};


int Billiard::ballCounter = 0;


int main(int argc, char *argv[]){
  int rank, size;

  srand((unsigned)time(0));
  Sector s = Sector(Coordinate(1,1), Coordinate(30,30));
  bool br=false;
  for( int i = 0; i < 20; i++){
    Velocity v = Velocity(Util::vGenerator1(2, 6), Util::aGenerator(1, 360));
    Coordinate c = Coordinate(Util::pGenerator1(1, 30), Util::pGenerator1(1,30));
    Billiard* pb = new Billiard(c, v);
    br=s.addABilliard(pb);   
    if(!br){
      i--;
      delete pb;
    }    
  }

  Sector s1 = Sector(Coordinate(31,1), Coordinate(60,30));
  br=false;
  for( int i = 0; i < 20; i++){
    Velocity v = Velocity(Util::vGenerator1(2, 6), Util::aGenerator(1, 360));
    Coordinate c = Coordinate(Util::pGenerator1(31, 60), Util::pGenerator1(1,30));
    Billiard* pb = new Billiard(c, v);
    br=s1.addABilliard(pb);   
    if(!br){
      i--;
      delete pb;
    }    
  }
  
  s.neighborNextTime=s1.neighborNextTime=0.0;
  s.nStart=s1.coordStart;
  s.nEnd=s1.coordEnd;
  s1.nStart=s.coordStart;
  s1.nEnd=s.coordEnd;

  Event* initEvent = new Event(0.0, Event::Initial, NULL);
  for (vector<Billiard*>::iterator p = s.ballList.begin(); p!=s.ballList.end(); p++){
    Billiard *bp = new Billiard();
    bp->id = (*p)->id;
		bp->v = (*p)->v;
    bp->coord = (*p)->coord;
    initEvent->ballList.push_back(bp);
  }
  s.eventLog.push_back(initEvent);

  Event* initEvent1 = new Event(0.0, Event::Initial, NULL);
  for (vector<Billiard*>::iterator p = s1.ballList.begin(); p!=s1.ballList.end(); p++){
    Billiard *bp = new Billiard();
    bp->id = (*p)->id;
		bp->v = (*p)->v;
    bp->coord = (*p)->coord;
    initEvent1->ballList.push_back(bp);
  }
  s1.eventLog.push_back(initEvent1);


	MPI_Init(&argc, &argv);
	rank=MPI::COMM_WORLD.Get_rank();
	size=MPI::COMM_WORLD.Get_size();

  if(rank==0){
    //fstream fcout("./sector0.txt",ios::out);
    double eventReceived[7];
    double eventSent[7];
		double previousTime=0.0;
    MPI::Request req;
		s.nextEvent=s.getNextEvent();
    s.nextTime=s.nextEvent->timeStamp;
		int i = 0;

    while(i < 1000){
      if(s.nextEvent->type==Event::Departure){
        eventSent[0]=1.0;   //type of the message
        eventSent[1]=s.nextEvent->timeStamp;
        eventSent[2]=s.nextEvent->ball->id;
        eventSent[3]=s.nextEvent->ball->coord.x+s.nextEvent->ball->v.vx*(s.nextTime-s.currentTime);
        eventSent[4]=s.nextEvent->ball->coord.y+s.nextEvent->ball->v.vy*(s.nextTime-s.currentTime);
        eventSent[5]=s.nextEvent->ball->v.vx;
        eventSent[6]=s.nextEvent->ball->v.vy;
        s.remove(s.nextEvent->ball);
      }
      else{
          eventSent[0]=0.0;
          eventSent[1]=s.nextEvent->timeStamp;
      }
      s.mpiSend(eventSent, 7, MPI_DOUBLE, 1, 19);
      s.mpiRecv(eventReceived, 7, MPI_DOUBLE, 1, 17, s.status);
			if(eventReceived[1]-previousTime != 0.0){
        if(eventReceived[0]-1.0==0.0){
	  			bool exist = false;
          if(s.exEvents.size()>0){
	  			  for (vector<Event*>::iterator p = s.exEvents.begin(); p!=s.exEvents.end(); p++){
	  				  if(((*p)->timeStamp-eventReceived[1]) == 0.0){
	  						exist = true;
	  						break;
	  					}
	  				}
	  			}
	  			if(!exist){
            Velocity v = Velocity();
            v.vx=eventReceived[5];
            v.vy=eventReceived[6];
            Coordinate c = Coordinate(eventReceived[3]-1, eventReceived[4]);
            Billiard* bp = new Billiard(c, v);
            bp->id = (int)eventReceived[2];
            double dt = eventReceived[1]+fabs(1.0/eventReceived[5]);
            Event* arrEvent = new Event(dt, Event::Arrival, bp);
            s.exEvents.push_back(arrEvent);
	  			}
        }
				previousTime = eventReceived[1];
        s.neighborNextTime=eventReceived[1];
			}

      if(s.nextTime<s.neighborNextTime){
        s.computeNewPositions();
        s.currentTime+=s.nextEvent->timeStamp-s.currentTime;
        s.currentEvent=s.nextEvent;
				s.nextEvent=s.getNextEvent();
	      s.nextTime=s.nextEvent->timeStamp;
      }

      i++;
    }
    //cout << "External Events in S0" << s.exEvents.size() << endl;
    s.saveEventLog("sector0.xml", "Billiard Simulation", "Sector0");
  }
  else if(rank==1){
    double eventReceived[6];
    double eventSent[6];
		double previousTime=0.0;
    MPI::Request req;
		s1.nextEvent=s1.getNextEvent();
    s1.nextTime=s1.nextEvent->timeStamp;
    int i = 0;
    while(i<1000){
      s1.mpiRecv(eventReceived, 7, MPI_DOUBLE, 0, 19, s1.status);      
			if(eventReceived[1]-previousTime != 0.0){
        if(eventReceived[0]-1.0==0.0){
    		  bool exist = false;
          if(s1.exEvents.size()>0){
				    for (vector<Event*>::iterator p = s1.exEvents.begin(); p!=s1.exEvents.end(); p++){
				  	  if(((*p)->timeStamp-eventReceived[1]) == 0.0){
				  			exist = true;
				  			break;
				  		}
				  	}
				  }
				  if(!exist){
            Velocity v = Velocity();
            v.vx=eventReceived[5];
            v.vy=eventReceived[6];
            Coordinate c = Coordinate(eventReceived[3]+1, eventReceived[4]);
            Billiard* bp = new Billiard(c, v);
            bp->id = (int)eventReceived[2];
            double dt = eventReceived[1]+fabs(1.0/eventReceived[5]);
            Event* arrEvent = new Event(dt, Event::Arrival, bp);
            s1.exEvents.push_back(arrEvent);
				  }
        }
				previousTime=eventReceived[1];
		    s1.neighborNextTime=eventReceived[1];
			}

      if(s1.nextEvent->type==Event::Departure){
        eventSent[0]=1.0;
        eventSent[1]=s1.nextEvent->timeStamp;
        eventSent[2]=s1.nextEvent->ball->id;
        eventSent[3]=s1.nextEvent->ball->coord.x+s1.nextEvent->ball->v.vx*(s1.nextTime-s1.currentTime);
        eventSent[4]=s1.nextEvent->ball->coord.y+s1.nextEvent->ball->v.vy*(s1.nextTime-s1.currentTime);
        eventSent[5]=s1.nextEvent->ball->v.vx;
        eventSent[6]=s1.nextEvent->ball->v.vy;
        s1.remove(s1.nextEvent->ball);
      }
      else{
        eventSent[0]=0.0;
        eventSent[1]=s1.nextEvent->timeStamp;
      }
      s1.mpiSend(eventSent, 7, MPI_DOUBLE, 0, 17);

      if(s1.nextTime<s1.neighborNextTime){
        s1.computeNewPositions();
        s1.currentTime+=s1.nextEvent->timeStamp-s1.currentTime;
        s1.currentEvent=s1.nextEvent;
	    	s1.nextEvent=s1.getNextEvent();
        s1.nextTime=s1.nextEvent->timeStamp;
      }     

      i++;
    }
    //cout << "External Events in S1" << s1.exEvents.size() << endl;
    s1.saveEventLog("sector1.xml", "Billiard Simulation", "Sector1");
  }
	MPI::Finalize();
	return 0;
}
