#include "sector.h"
#include <fstream>

bool Sector::addABilliard(Billiard* pBall){
  vector<Billiard*>::iterator p = ballList.begin();
  //check if there is a ball at the same location
  while(p!=ballList.end()){
    if(((*p)->coord.x==pBall->coord.x) && ((*p)->coord.y==pBall->coord.y)){
      //cout << "There is already a ball in this location ..." << endl;
      return false;
    }
    else      
      p++;
  }
  ballList.push_back(pBall);
  return true;
}

void Sector::print(){
  for (vector<Billiard*>::iterator p = ballList.begin(); p!=ballList.end(); p++)
    (*p)->print();
}

double Sector::collisionTime(const Billiard* bpfrom, const Billiard* bpto ){
  double distx = bpto->coord.x - bpfrom->coord.x;
  double disty = bpto->coord.y - bpfrom->coord.y;
  
  double vx = bpto->v.vx - bpfrom->v.vx;
  double vy = bpto->v.vy - bpfrom->v.vy;

  double timex;
  if(vx==0){timex = INF;}
  else{
    timex = -distx/vx;
    if(timex<=0.1)
      timex=INF;
  }
  double timey;
  if(vy==0){timey = INF;}
  else{
		timey = disty/vy;
    if(timey<=0.1)
			timey=INF;
  }

  if(fabs(timex-timey) < 0.5 && timex!=INF){
    //cout << "Billiard " << bpfrom->id << " and " << bpto->id << " : ";
    //cout << "Distance: (" << distx << " , " << disty << ") "; 
    //cout << " Velocity: ( " << vx << " , " << vy << " )";
    //cout << " CollisionTime: ( " << timex << " , " << timey << " )" << endl;
    return timex;
  }
  return INF; 
}

double Sector::bounceTime(const Billiard* bp){
  double timex = 0.0;
  double timey = 0.0;

	if(bp->v.vx == 0){
    timex = INF;
  }
  else{
    if(bp->v.vx > 0){
      timex = (coordEnd.x - bp->coord.x)/bp->v.vx;
    }
    else{
      timex = (coordStart.x - bp->coord.x)/bp->v.vx;
    }
  }

  if(timex<=0.1){ timex=INF;}
	if(bp->v.vy == 0){
    timey = INF;
  }
  else{
    if(bp->v.vy > 0){
      timey = -(coordStart.y - bp->coord.y)/bp->v.vy;
    }
    else{
      timey = (bp->coord.y - coordEnd.y)/bp->v.vy;
    }
  }
  if(timey<=0.1){timey=INF;}
  //cout << "Billiard " << bp->id << " bounce time: " << timex << " , " << timey << " )" << endl;
  return timex < timey ? timex : timey;
  
}

Event* Sector::getNextCollisionEvent(){
  Event* minEvent = new Event(currentTime+INF, Event::Collision, NULL);
  double minTime = currentTime+INF;
  vector<Billiard*>::iterator pHead = ballList.begin();
  vector<Billiard*>::iterator p = pHead;

  while(pHead != ballList.end()){
    p++;
    while(p!=ballList.end()){
      //minTime = currentTime+(*pHead)->collisionTime(*p);
			minTime = currentTime+collisionTime(*pHead, *p);
      //cout << minTime << endl;
      if(minTime < minEvent->timeStamp){
        minEvent->timeStamp = minTime;
        minEvent->ball = *pHead;
        minEvent->ball1 = *p;
  			//(*pHead)->print();
				//(*p)->print();
			}
      p++;
    }
  	pHead ++;
    p = pHead;
  }
  return minEvent;
}

Event* Sector::getNextBounceEvent(){
  Event* minEvent = new Event(currentTime+INF, Event::Bounce, NULL);
  double minTime = currentTime+INF;

  vector<Billiard*>::iterator p = ballList.begin();
  while(p!=ballList.end()){        
    minTime=currentTime+bounceTime(*p);
    //cout << "bounce event: " << minTime << endl; 
		if(minTime < minEvent->timeStamp){
			minEvent->timeStamp=minTime;
			minEvent->ball = *p;
		}
    p++;
  }
  if(isToNeighbor(minEvent->ball, minEvent->timeStamp-currentTime)){
		//cout<< "Departure Event .... " << minEvent->ball->id <<  endl;
		minEvent->type=Event::Departure;
  }
	return minEvent;
}

Event* Sector::getNextEvent(){
  Event* collEvent = getNextCollisionEvent();
  Event* bounEvent = getNextBounceEvent();
  Event* arrEvent = NULL;
  if(collEvent->timeStamp<=bounEvent->timeStamp){
    if(exEvents.size()>0 && exEvents[0]->timeStamp<=collEvent->timeStamp){
      delete bounEvent;
      delete collEvent;
      arrEvent=exEvents[0];
      exEvents.erase(exEvents.begin());
      return arrEvent;
    }
    else{
      delete bounEvent;
      return collEvent;
    }
  }
  else{
    if(exEvents.size()>0 && exEvents[0]->timeStamp<=bounEvent->timeStamp){
      delete collEvent;
      delete bounEvent;
      arrEvent=exEvents[0];
      exEvents.erase(exEvents.begin());
      return arrEvent;
    }
    else{
      delete collEvent;
      return bounEvent;
    }
  }
}

void Sector::computeNewPositions(){
  double tDiff;
  if(currentEvent!=NULL){
	  tDiff=nextEvent->timeStamp - currentEvent->timeStamp;
    //cout << "time difference : " << tDiff << endl;
  }
  else{
	  tDiff=nextEvent->timeStamp;
    //cout << "time difference : " << tDiff << endl;
  }

  vector<Billiard*>::iterator p = ballList.begin();
  while(p!=ballList.end()){
    (*p)->coord.x=(*p)->coord.x+(*p)->v.vx*tDiff;
    (*p)->coord.y=(*p)->coord.y-(*p)->v.vy*tDiff;
    if(((*p)->coord.x-coordStart.x)<1)
      (*p)->coord.x=coordStart.x;
    if((coordEnd.x-(*p)->coord.x)<1)
      (*p)->coord.x=coordEnd.x;
    if(((*p)->coord.y-coordStart.y)<1)
      (*p)->coord.y=coordStart.y;
    if((coordEnd.y-(*p)->coord.y)<1)
      (*p)->coord.y=coordEnd.y;
    p++;
  }
  //post processing
  if(nextEvent->type == Event::Collision){
      //cout<< "collision evnet postprocessing ...." << endl;
      //nextEvent->print();
      Velocity vt;
      vt=nextEvent->ball->v;
      nextEvent->ball->v=nextEvent->ball1->v;
      nextEvent->ball1->v=vt;
      //nextEvent->ball->exchangeVelocity(nextEvent->ball1);
      nextEvent->ball->v.computeAngle();
      nextEvent->ball1->v.computeAngle();
  }

  if(nextEvent->type == Event::Arrival){
		addABilliard(nextEvent->ball);
  }

  for (vector<Billiard*>::iterator p = ballList.begin(); p!=ballList.end(); p++){
    Coordinate cood = (*p)->coord;
      //cood.print();    
    if(fabs(cood.x-coordStart.x)<0.1 || fabs(cood.x-coordEnd.x)<0.1)
      (*p)->v.vx=0-(*p)->v.vx;
    if(fabs(cood.y-coordStart.y)<0.1 || fabs(cood.y-coordEnd.y)<0.1)
      (*p)->v.vy=0-(*p)->v.vy;
	}

/*
  else if(nextEvent->type == Event::Departure){
  }
  else{ //Event::Bounce
    Coordinate cood = nextEvent->ball->coord;
    //cood.print();    
    if(fabs(cood.x-coordStart.x)<0.1 || fabs(cood.x-coordEnd.x)<0.1)
      nextEvent->ball->v.vx=0-nextEvent->ball->v.vx;
    if(fabs(cood.y-coordStart.y)<0.1 || fabs(cood.y-coordEnd.y)<0.1)
      nextEvent->ball->v.vy=0-nextEvent->ball->v.vy;
    nextEvent->ball->v.computeAngle();
  }
*/
//  if(currentEvent!=NULL){
  for (vector<Billiard*>::iterator p = ballList.begin(); p!=ballList.end(); p++){
    Billiard *bp = new Billiard();
    bp->id = (*p)->id;
		bp->v = (*p)->v;
    bp->coord = (*p)->coord;
    nextEvent->ballList.push_back(bp);
  }
  eventLog.push_back(nextEvent);
//  }

//  currentEvent=nextEvent;
//  nextEvent=NULL;
}

void Sector::run(double howLong){
  currentEvent=NULL;
  nextEvent=getNextEvent();
  //nextEvent=getNextCollisionEvent();
  //print();
  nextTime=nextEvent->timeStamp;
  double i=0.0;
  int j=0;
  while(i < howLong){
    //cout << "current Time: " << currentTime << endl;
    //print();
    //nextEvent->print();
    //nextEvent->ball->print();
    //if(nextEvent->type==Event::Collision){
    //  cout << "current Time: " << currentTime << endl;
    //  nextEvent->print();
    //  nextEvent->ball->print();
    //  nextEvent->ball1->print(); 
    //}
    computeNewPositions();
    //nextEvent->ball->print();
    if(nextEvent->type==Event::Collision){
      cout<<j++<<endl;
      nextEvent->print();
      //nextEvent->ball->print();
    //  nextEvent->ball1->print(); 
    }
    //print();
    i+= 1.0;
    currentTime+=nextEvent->timeStamp-currentTime;
    currentEvent=nextEvent;
    //currentTime=currentEvent->timeStamp;
    //currentEvent->print();
    nextEvent=getNextEvent();
    //nextEvent=getNextCollisionEvent();
    nextTime=nextEvent->timeStamp;
 }
}

void Sector::collisionTest(){
  vector<Billiard*>::iterator p = ballList.begin();
  vector<Billiard*>::iterator pOld = p;
  p++;
  //int i = 0;
  while(p!=ballList.end()){
    collisionTime(*pOld, *p);
    //cout << "Collision Angle " << (*pOld)->id << ":" << (*pOld)->angle(*(*p));
    //cout << " at velocity: "<< (*pOld)->collisionVelocity(*(*p)) << endl;    
    //cout << "Collision Angle " << (*p)->id << ":" << (*p)->angle(*(*pOld)); //<< endl; 
    //cout << " at velocity: "<< (*p)->collisionVelocity(*(*pOld))<< endl;
    //cout << "Collision time " << (*pOld)->id << ":" << (*pOld)->collisionTime(*p)<< endl; 
    pOld=p;
    p++;
  }
}

bool Sector::isToNeighbor(const Billiard* b, double dTime){
  double x=b->coord.x+b->v.vx*dTime;
  if(( (x+1) >= nStart.x && (x+1) <= nEnd.x)
    || ( (x-1) >= nStart.x && (x-1)<=nEnd.x))
    return true;
  else 
		return false;
}

void Sector::remove(Billiard* b){
  vector<Billiard*>::iterator p = ballList.begin();
  while(p!=ballList.end()){
    if((*p)->id==b->id){
      dupBalls.push_back(b);
      ballList.erase(p);
      break;
    }
    p++;
  }
}

void Sector::mpiSend(void *buf, int count, MPI_Datatype type, int source, int tag){
	MPI::COMM_WORLD.Send(buf, count, type, source, tag);
}

void Sector::mpiRecv(void *buf, int count, MPI_Datatype type, int source, int tag, MPI::Status status){
	MPI::COMM_WORLD.Recv(buf, count, type, source, tag, status);
}

MPI::Request Sector::mpiISend(void *buf, int count, MPI_Datatype type, int source, int tag){
  return	MPI::COMM_WORLD.Isend(buf, count, type, source, tag);
}

MPI::Request Sector::mpiIRecv(void *buf, int count, MPI_Datatype type, int source, int tag){
	MPI::COMM_WORLD.Irecv(buf, count, type, source, tag);
}


void Sector::saveEventLog(char* fileName, char* projectName, char* componentName){

    fstream fcout(fileName,ios::out);
    fcout << "<?xml version=\"1.0\"?>" << endl;
    fcout << "<trace project=\"" << projectName << "\" component=\"" << componentName <<"\">" << endl;
    fcout << "<states>" << endl;
    fcout << "<state>Arrival</state>" << endl;
    fcout << "<state>Departure</state>" << endl;
    fcout << "<state>Moving</state>" << endl;
    fcout << "<state>Collision</state>" << endl;
    fcout << "<state>Bounce</state>" << endl;
    fcout << "</states>" << endl;
    vector<Event*>::iterator p = eventLog.begin();
    fcout<< "<events>" << endl;
//    fcout << "<event>" << endl;
//    fcout << "<time>" << "0.0" << "</time>" << endl;
//    fcout << "<deststate>" << "Moving" << "</deststate>" << endl;
//    fcout << "<content>" << "Intializing" << "</content>" << endl;
//    fcout << "<type>" << "IN" << "</type>" << endl;
//    fcout << "</event>" << endl;
    while(p!=eventLog.end()){
      fcout << "<event>" << endl;
      fcout << "<time>" << (*p)->timeStamp << "</time>" << endl;
      if((*p)->type==Event::Collision){
        fcout << "<deststate>" << "Collision" << "</deststate>" << endl;
        fcout << "<content>" << "Collision;ball  " << (*p)->ball->id << ";ball  " << (*p)->ball1->id << "</content>" << endl;
        fcout << "<type>" << "IN" << "</type>" << endl;
      }
      else if((*p)->type==Event::Departure){
        fcout << "<deststate>" << "Departure" << "</deststate>" << endl;
        fcout << "<content>" << "Departure;ball  " << (*p)->ball->id << "</content>" << endl;
        fcout << "<type>" << "IN" << "</type>" << endl;
      }
      else if((*p)->type==Event::Arrival){
        fcout << "<deststate>" << "Arrival" << "</deststate>" << endl;
        fcout << "<content>" << "Arrival;ball  " << (*p)->ball->id << "</content>" << endl;
        fcout << "<type>" << "IN" << "</type>" << endl;
      }
      else if((*p)->type==Event::Bounce){
        fcout << "<deststate>" << "Bounce" << "</deststate>" << endl;
        fcout << "<content>" << "Bounce;ball  " << (*p)->ball->id << "</content>" << endl;
        fcout << "<type>" << "IN" << "</type>" << endl;
      }
			else{
        fcout << "<deststate>" << "Moving" << "</deststate>" << endl;
        fcout << "<content>" << "Intializing" << "</content>" << endl;
        fcout << "<type>" << "IN" << "</type>" << endl;
			}

      fcout << "<ballstates>" << endl;
      vector<Billiard*>::iterator ep;
      for ( ep = (*p)->ballList.begin(); ep!=(*p)->ballList.end(); ep++){
        fcout << "<ball";
        fcout << " id=\"" << (*ep)->id << "\"";
        fcout << " v=\"" << (*ep)->v.v << "\"";
        fcout << " angle=\"" << (*ep)->v.angle << "\"";  
        fcout << " x=\"" << (*ep)->coord.x << "\"";  
        fcout << " y=\"" << (*ep)->coord.y << "\"";
        fcout << "></ball>" << endl;  
      }
      fcout  << "</ballstates>" << endl;
      fcout << "</event>" << endl;
/*      fcout << "<event>" << endl;
      fcout << "<time>" << (*p)->timeStamp << "</time>" << endl;
      fcout << "<deststate>" << "Moving" << "</deststate>" << endl;
      fcout << "<content>" << "Finished" << "</content>" << endl;
      fcout << "<type>" << "IN" << "</type>" << endl;
      fcout << "</event>" << endl;
*/
      p++;
    }    
    fcout<< "</events>" << endl;
    fcout<< "</trace>"  << endl;
    fcout.close();
}

Sector::~Sector(){
  for (vector<Billiard*>::iterator p = ballList.begin(); p!=ballList.end(); p++)
    delete *p;

  for (vector<Event*>::iterator p = eventLog.begin(); p!=eventLog.end(); p++)
    delete *p;

  //for (vector<Event*>::iterator p = exEvents.begin(); p!=exEvents.end(); p++)
  //  delete *p;

  //for (vector<Billiard*>::iterator p = dupBalls.begin(); p!=dupBalls.end(); p++)
  //  delete *p;

  //if(Event!=NULL) delete lastEvent;
  //if(nextEvent!=NULL) delete nextEvent;
}
