#include <vector>
#include <mpi.h>
#include "datastructure.h"

class Area{
  public:
    Coordinate coordStart;
    Coordinate coordEnd;

    Area(){}
    Area(Coordinate start, Coordinate end){
      coordStart = start;
      coordEnd = end;
    }
    Area(const Area& src){
      coordStart = src.coordStart;
      coordEnd   = src.coordEnd;
    }
};


class Sector:public Area{
  public:  
    vector<Billiard*> ballList;
    vector<Billiard*> dupBalls;
    vector<Sector*>   neighbors;
    vector<Event*>    eventLog;
    vector<Event*>    exEvents;  //save the arrived balls
    Event* currentEvent;
		Event* nextEvent;
    double currentTime;
    double nextTime;
    double neighborNextTime;
    Coordinate nStart;  //Start coordinate of neighbor
    Coordinate nEnd;    //End coordinate of neighbor
    MPI::Status status;
    MPI::Request request;

    Sector(){ 
			currentEvent=NULL;
      currentTime=0.0;
      nextTime=0.0;
 			nextEvent=NULL;
		}

    Sector(Coordinate start, Coordinate end):Area(start,end){
			currentEvent=NULL;
      currentTime=0.0;
      nextTime=0.0;
			nextEvent=NULL;
		}

    bool   addABilliard(Billiard* pBall);
    void   addANeighbor(Sector* pSector);
    void   print();
    double collisionTime(const Billiard* bpfrom, const Billiard* bpto );
    double bounceTime(const Billiard* bp);
    Event* getNextCollisionEvent();
    Event* getNextBounceEvent();
    Event* getNextEvent();
    void   computeNewPositions();
    void   run(double howLong);
    void   collisionTest();
    bool   isToNeighbor(const Billiard* b, double dTime);
    void   remove(Billiard* b);
    void   mpiSend(void *buf, int count, MPI_Datatype type, int source, int tag);
    void   mpiRecv(void *buf, int count, MPI_Datatype type, int source, int tag, MPI::Status status);
    MPI::Request mpiISend(void *buf, int count, MPI_Datatype type, int source, int tag);
    MPI::Request mpiIRecv(void *buf, int count, MPI_Datatype type, int source, int tag);
    void   saveEventLog(char* fileName, char* projectName, char* componentName);
    // computeCollision();
    ~Sector();

    //Event* nextOtherEvent(){ //next bounce or departure event
    //  Event evt;
    //  int bTime = 100000;
    //  vector<Billiard*>::iterator p = ballList.begin();
    //  while(p!=ballList.end()){
        
    //    canCollide(*pOld, *p);
    //    pOld=p;
    //    p++;
    //  }
    //}


};
